;*********************************************************************
;Curso de AutoLISP
;Rotinas teis
;por Denio Guimares - Niteri, RJ
;www.cadnorma.com.br
;cadnorma@cadnorma.com.br
;*********************************************************************


;Este arquivo no possui funes especficas, apenas trechos de programas
;que sero teis na confeco de rotinas. No deve ser carregado diretamente no AutoCAD


;Algumas das funes usam caixas de dilogo que esto definidas no Arquivo
;Cadnorma.DCL, parte integrande do Cadnorma (INPI 049.844/2003), incluso no DVD do Curso de AutoLISP



;*********************************************************************
;funes de erro                                                      
;*********************************************************************
(defun guarda (VS / VI value)
  (setq err_temp *ERROR*)
  (setq *error* armadilha)
  (setq restlist '())
   (while VS
      (setq VI (car VS)
            value (getvar VI)
            restlist (append restlist (list (list VI value)))
            VS (cdr VS)
      )
   )
)

(defun devolve ()  
  (setq *error* err_temp)
  (while restlist
     (setq VI (car (car restlist))
           value (car (cdr (car restlist)))
     )
     (setvar VI value)
     (setq restlist (cdr restlist))
  )
)


(defun armadilha (errmsg) 
  (command nil nil nil)
  (if (not
	(member errmsg
	   '("console break" "Function Cancelled"))
      )
    (princ (strcat "\nError: " errmsg))
)
(princ)
(terpri)
(setq *error* err_temp)
(princ)
)

(defun *error* (msg)
  (alert (strcat "OCORREU ERRO " MSG))
  (devolve)
  (setq *error* %erro)
)


;*********************************************************************
;  Mensagens de Alerta com DCL`s personalizadas **********************
;*********************************************************************

;somente com o boto "ok"
(defun cn7_alert_ok (msg1 msg2 msg3)
  (setq alert_resp nil)
  (setq dcl_id (load_dialog (strcat cnp_drive ":/cadnorma/lisp/cadnorma.dcl")))
  (if (not (new_dialog "cn7_msg_ok" dcl_id)) (exit))
  (set_tile "msg_text1" msg1)
  (set_tile "msg_text2" msg2)
  (set_tile "msg_text3" msg3)
  (action_tile "accept" " (done_dialog 1)")
  (setq msg_resp (start_dialog))
  (unload_dialog dcl_id)
  (princ)
)

;com os botes "ok" e "cancel"
(defun cn7_alert_ok_cancel (msg1 msg2 msg3)
  (setq alert_resp nil)
  (setq dcl_id (load_dialog (strcat cnp_drive ":/cadnorma/lisp/cadnorma.dcl")))
  (if (not (new_dialog "cn7_msg_ok_cancel" dcl_id)) (exit))
  (set_tile "msg_text1" msg1)
  (set_tile "msg_text2" msg2)
  (set_tile "msg_text3" msg3)
  (action_tile "accept" " (done_dialog 1)")
  (setq msg_resp (start_dialog))
  (unload_dialog dcl_id)
  (if (= msg_resp 1) (setq alert_resp "ok") (setq alert_resp "cancel"))
  (princ)
)


;com os botes "Sim", "No"
(defun cn7_alert_yes_no (msg1 msg2 msg3)
  (setq alert_resp nil)
  (setq dcl_id (load_dialog (strcat cnp_drive ":/cadnorma/lisp/cadnorma.dcl")))
  (if (not (new_dialog "cn7_msg_yes_no" dcl_id)) (exit))
  (set_tile "msg_text1" msg1)
  (set_tile "msg_text2" msg2)
  (set_tile "msg_text3" msg3)
  (action_tile "yes" " (done_dialog 1)")
  (setq msg_resp (start_dialog))
  (unload_dialog dcl_id)
  (if (= msg_resp 1) (setq alert_resp "yes") (setq alert_resp "no"))
  (princ)
)

;com os botes "Sim", "No" e Cancela
(defun cn7_alert_yes_no_cancel (msg1 msg2 msg3)
  (setq alert_resp nil)
  (setq dcl_id (load_dialog (strcat cnp_drive ":/cadnorma/lisp/cadnorma.dcl")))
  (if (not (new_dialog "cn7_msg_yes_no_cancel" dcl_id)) (exit))
  (set_tile "msg_text1" msg1)
  (set_tile "msg_text2" msg2)
  (set_tile "msg_text3" msg3)
  (action_tile "yes" "(done_dialog 1)")
  (action_tile "no" "(done_dialog 2)")
  (setq msg_resp (start_dialog))
  (unload_dialog dcl_id)
  (if (= msg_resp 1)
    (setq alert_resp "yes")
    (progn
      (if (= msg_resp 2)
	(setq alert_resp "no")
	(setq alert_resp "cancel")
      )
    )
  )
  (princ)
)


;*********************************************************************
;  Carregar e Descarregar menus dentro do AutoCAD*********************
;*********************************************************************


;Rotina para carergar o menu do Cadnorma dentro do AutoCAD
(command "_menuunload" "cadnorma_p")
(command "_menuload" (strcat cnp_drive ":/cadnorma/menus/cnp_autocad_" cnp_drive ".mnu"))
(menucmd "P18=+cadnorma_p.cadnorma_p")

;funo que descarrega o Cadnorma_P
(defun c:cn_unload ()
  (cn7_alert_yes_no
    "Este comando ir descarregar o menu"
    "do Cadnorma Pocket!"
    "Deseja continuar?"
  )
  (if (= alert_resp "yes")
    (progn
      (command "_menuunload" "cadnorma_P")
      (cn7_alert_ok
	"Cadnorma Professional descarregado..."
	"Use o comando 'appload' para"
	"Carregar o menu novamente."
      )
    )
  )
  (princ)
)


;*********************************************************************
;  Escolher a cor dentro de uma DCL **********************************
;*********************************************************************

;esta funo  definida como o "action_tile" de uma imagem (icon_image)
;voc clica na imagem e ele permite selecionar uma cor para atribuir a uma varivel

;escolher cor do texto
(defun aldim_txcol ()
  (setq aldim_newcol (acad_colordlg (fix (distof aldim_tcol))))
  (if (/= nil aldim_newcol) (setq aldim_tcol (rtos aldim_newcol 2 0)))
  (start_image "aldim_txt_color")
  (fill_image  0  0
    (dimx_tile "aldim_txt_color")
    (dimy_tile "aldim_txt_color")
    (fix (distof aldim_tcol))
  )
  (end_image)
)

;******************************************************************************
;Funo que verifica o status do layer requisitado:
;se no existir, ser criado!!!
;******************************************************************************

;requer dois parmetros: nome do layer e cor do layer

(defun cn7_verylayer (cn7_very_layname cn7_laycol)
  (setq cn7_currlayer nil)
  ;se existe o layer solicitado
  (if (/= nil (setq cn7_currlayer (tblsearch "layer" cn7_very_layname)))
    (progn
      (setq cn7_layname (cdr (assoc 2 cn7_currlayer)))
      ;verifica se est desligado
      (if (= -30 (cdr (assoc 62 cn7_currlayer)))
	(progn
	  (cn7_alert_yes_no
	    "O layer solicitado est desligado!"
	    (strcat "layer = " cn7_layname)
	    "Deseja lig-lo e Prosseguir?"
	  )
	  (if (= alert_resp "yes")
	    (command "._layer" "_on" cn7_layname "")
	    (exit)
	  )
	)
      );if
      (cond
	;se (assoc 70)  1, est s freeze
	((= 1 (cdr (assoc 70 cn7_currlayer)))
	 (cn7_alert_yes_no
	   "O layer solicitado est congelado"
	   (strcat "layer = " cn7_layname)
	   "Deseja descongel-lo e Prosseguir?"
	 )
	 (if (= alert_resp "yes")
	   (command "._layer" "_thaw" cn7_layname "")
	   (exit)
	 )
        )
	;se (assoc 70)  4,  s lock
	((= 4 (cdr (assoc 70 cn7_currlayer)))
	 (cn7_alert_yes_no
	   "O layer solicitado est trancado"
	   (strcat "layer = " cn7_layname)
	   "Deseja destranc-lo e Prosseguir?"
	 )
	 (if (= alert_resp "yes")
	   (command "._layer" "_unlock" cn7_layname "")
	   (exit)
	 )
        )
	;se (assoc 70)  5, est freeze e lock
	((= 5 (cdr (assoc 70 cn7_currlayer)))
	 (cn7_alert_yes_no
	   "O layer solicitado est trancado e congelado!"
	   (strcat "layer = " cn7_layname)
	   "Deseja desfazer e Prosseguir?"
	 )
	 (if (= alert_resp "yes")
	   (progn
	     (command "._layer" "_unlock" cn7_layname "")
	     (command "._layer" "_thaw" cn7_layname "")
	   )
	   (exit)
	 )
        )
      );cond
      (command "._layer" "_m" cn7_very_layname "_c" cn7_laycol "" "")
    )
    (progn
      (command "._layer" "_m" cn7_very_layname "_c" cn7_laycol "" "")
      (prompt (strcat "...Criado layer <" (strcase cn7_very_layname) ">, com a cor <" (rtos cn7_laycol 2 0) ">."))
    )
  )
)


;******************************************************************************
;Funo que verifica se a entrada em uma list_box  um nmero real
;esta funo  um "action_tile" associada a uma "tile" tipo "edit_box"
;******************************************************************************

;Verifica dado Largura
(defun cn7_lim_verilarg ()
  (setq cn7_cfg_limits_largura $value)
  (if (= nil (distof cn7_cfg_limits_largura))
    (progn
      (alert "A largura deve ser um nmero real!\nTente novamente...")
      (mode_tile "cn7_cfg_limits_largura" 2)
    )
  )
)



;*********************************************************************
;funo que encontra a escala da viewport ativa
;*********************************************************************
(defun cn7_vport_scl ()
  (setq cn7_ddzin (getvar "dimzin"))
  (setvar "dimzin" 1)
  (if (not (and (= 0 (getvar "tilemode")) (= 1 (getvar "cvport"))))
    (setq scl_list (assoc (getvar "cvport") (setq all_views (vports)))
	  scl_delta (-  (nth 1 (nth 2 scl_list)) (nth 1 (nth 1 scl_list)))
	  cn7_vp_scale (/ (* 1000 (/ (getvar "viewsize") scl_delta)) cn7_dwgunit);next->
    )
    (alert "no est na janela")
  )
  (setvar "dimzin" cn7_ddzin)
  (setq cn10_vp_scale (* 1000 (/ (getvar "viewsize") scl_delta)))
)


;******************************************************************************
;******************************************************************************
;Ferramentas do Paper Space
;******************************************************************************
;******************************************************************************

;tranca as janelas selecionadas
(defun c:vp_lock ()
  (guarda '("blipmode" "cecolor" "celtype" "clayer" "cmdecho" "orthomode"
          "textstyle" "luprec" "osmode" "dimzin"))
  (if (= 1 (getvar "tilemode"))
    (progn
      (cn7_alert_ok
	"Voc est operando no Model Space."
	"Esta operao no  permitida aqui!"
	"Mude para o Paper Space...")
    (exit)
    )
  )
  (command "._pspace")
  (prompt "\nSelecione as Viewports que deseja trancar...")
  (setq cn7_vp_tolock (ssget '((0 . "VIEWPORT"))))
  (command "._mview" "_lock" "_on" cn7_vp_tolock "")
  (devolve)
  (princ)
)

;destranca as janelas selecionadas
(defun c:vp_unlock ()
  (if (= 1 (getvar "tilemode"))
    (progn
      (cn7_alert_ok
	"Voc est operando no Model Space."
	"Esta operao no  permitida aqui!"
	"Mude para o Paper Space...")
    (exit)
    )
  )
  (command "._pspace")
  (prompt "\nSelecione as Viewports que deseja destrancar...")
  (setq cn7_vp_tolock (ssget '((0 . "VIEWPORT"))))
  (command "._mview" "_lock" "_off" cn7_vp_tolock "")
  (princ)
)


;*****************************
;Funes de Layers
;*****************************


;pick layer (antigo change layer)
(defun c:pkl (/ p x s1)
  (setq x (entsel "\nSelecione a Entidade Base"))
  (setq p (cdr (assoc 8(entget (car x)))))
  (prompt "\nLayer base :")
  (prompt p)
  (prompt "\nSelecione as entidades para mudanca")
  (setq s1 (ssget))
  (command "._chprop" s1 "" "_la" p "")
  (princ)
)

;torna currente o layer de uma entidade selecionada
(defun c:cur (/ p x s1)
  (setq x (entsel "\nSelecione a cujo layer deve ser o atual..."))
  (setq p (cdr (assoc 8(entget (car x)))))
  (setvar "clayer" p)
  (princ)
)


;muda os objetos selecionados para o layer current
(defun C:tcr ()
  (setq a (getvar "clayer"))
  (prompt "\nSelecione as entidades para mudanca")
  (ssget)
  (command "._chprop" "_p" "" "_la" a "")
  (princ)
)

;Congela o layer de uma entidade selecionada
(defun c:fz (/ p x OorF)
  (while (/= nil (setq x (entsel "\nSelecione o objeto cujo layer deve ser congelado... ")))
    (setq fz_p (cdr (assoc 8 (entget (car x)))))
    (if (/= fz_p (getvar "clayer"))
      (command "._layer" "_freeze" fz_p "")
      (progn
	(alert "Impossvel Congelar o Layer Atual...
	\nSelecione uma Entidade para tornar seu Layer Corrente
	\ne Congelar o Layer Desejado...")
	(setq x2 (entsel))
	(setq p2 (cdr (assoc 8 (entget (car x2)))))
	(command "._layer" "_s" p2 "")
	(command "._layer" "_freeze" fz_p "")
      )
    )
  )
  (princ)
)

;desliga o layer selecionado
(defun c:ofl (/ p x OorF)
  (while (/= nil (setq x (entsel "\nSelecione o objeto cujo layer deve ser desligado... ")))
    (setq fz_p (cdr (assoc 8 (entget (car x)))))
    (if (/= fz_p (getvar "clayer"))
      (command "._layer" "_off" fz_p "")
      (progn
	(Initget 1 "Sim Nao")
	(setq resp (getkword "\nDeseja realmente desligar o layer atual? <S/N>... "))
	(if (= resp "Nao")
	  (progn
	    (prompt "Selecione uma entidade para tornar seu layer corrente... ")
	    (setq x2 (entsel))
	    (setq p2 (cdr (assoc 8 (entget (car x2)))))
	    (command "._layer" "_s" p2 "")
	    (command "._layer" "_off" fz_p "")
	    )
	  (progn
	    (command "._layer" "_off" fz_p "_y" "")
	    (alert "Ateno!\nLayer atual desligado!!!")
	    )
	);if
      )
    )
  )
  (princ)
)

;Tranca o layer de um conjunto de entidades selecionadas
(defun c:lkl (/ p x OorF)
  (prompt "\nSelecione os objetos cujos layers devem ser trancados <lock>")
  (setq s1 (ssget)
	s1_len (sslength s1)
	counter 0
	lay_list (list)
  )
  (command "._layer")
  (repeat s1_len
    (setq e1_lay (cdr (assoc 8 (entget (ssname s1 counter)))))
    (command  "_lo" e1_lay)
    (setq counter (1+ counter))
  )
  (command "")
  (princ)
)

;Destranca o layer de um conjunto de entidades selecionadas
(defun c:ulk (/ p x OorF)
  (prompt "\nSelecione os objetos cujos layers devem ser destrancados <unlock>")
  (setq s1 (ssget)
	s1_len (sslength s1)
	counter 0
	lay_list (list)
  )
  (command "._layer")
  (repeat s1_len
    (setq e1_lay (cdr (assoc 8 (entget (ssname s1 counter)))))
    (command  "_unlock" e1_lay)
    (setq counter (1+ counter))
  )
  (command "")
  (princ)
)


;descongela todos os layers
(defun c:twl () (command "._layer" "_t" "*" "") (princ))
(defun c:2 () (command "._layer" "_t" "*" "") (princ))


;liga todos os layers
(defun c:onl () (command "._layer" "_on" "*" "") (princ))

;destranca todos os layers
(defun c:ulka () (command "._layer" "_unlock" "*" "") (princ))



;******************************************************************************
;******************************************************************************
;Utilidades
;******************************************************************************
;******************************************************************************

;************ Funcao M2P *************
(defun c:m2p ()
  (setq p1 (getpoint "\nPrimeiro Ponto: "))
  (setq P2 (GETPOINT P1 "\nSegundo Ponto: "))
  (setq p3 (POLAR P1 (ANGLE P1 P2) (/ (DISTANCE P1 P2) 2)))
  (command "_non" p3)
  (princ)
)

;Le a cor de uma entidade base e copia para outras selecionadas
(defun c:cpc (/ p x s2)
  (prompt "Selecione a cor basica...")
  (setq e_color(cdr (assoc 62 (entget (car (entsel))))))
  (prompt "\nSelecione as Entidades a serem Transformadas")
  (setq s2 (ssget))
  (command "._chprop" s2 "" "_c" e_color "")
  (princ)
)

;Muda a cor e o tipo de linha de entidades selecionadas para BYLAYER
(defun c:by ()
  (prompt "\nSelecione as entidaddes para assumirem cor e tipo de linha BYLAYER...")
  (setq s1 (ssget))
  (command "._chprop" s1 "" "_c" "bylayer" "_lt" "bylayer" "")
  (princ)
);defun


;cnp_11


;Funcao MRT (Move-Rotate)
(defun c:mr (/ ss1 p1)
 (if (setq ss1 (ssget))
  (progn
   (setq p1 (getpoint "\nPonto base: "))
   (prompt "\nPonto destino: ")
   (command "._move" ss1 "" "_non" p1 "_non" p1 "._point" "_non" p1 "._move" ss1 (entlast) "" "_non" p1 pause)
   (prompt "\nngulo de rotao: ")
   (command "._rotate" ss1 "" (cdr (assoc 10 (entget (entlast)))) pause "._redraw")
  )
 )
 (command "._erase" "_l" "")
 (princ)
 )

;Funcao CR (Copy-Rotate)
(defun c:cr (/ ss1 p1)
 (prompt "\nSelecione os objetos a copiar e depois rotacionar... ")
 (if (setq ss1 (ssget))
  (progn
   (setq p1 (getpoint "\nPonto base: "))
   (prompt "\nPonto de destino...")
   (command "._copy" ss1 "" "_non" p1 "_non" p1 "._point" "_non" p1 "._move" ss1 (entlast) "" "_non" p1 pause)
   (prompt "\nngulo de rotao: ")
   (command "._rotate" ss1 "" (cdr (assoc 10 (entget (entlast)))) pause "._redraw")
  )
 )
 (command "._erase" "_l" "")
  (princ)
)

;fuo copy_scale
(defun c:csc (/ ss1 p1)
 (prompt "\nSelecione os objetos a copiar e depois escalar... ")
 (if (setq ss1 (ssget))
  (progn
   (setq p1 (getpoint "\nPonto base: "))
   (setq p2 (getpoint p1 "\nSegundo ponto ou <ENTER> para o mesmo ponto..."))
   (if (= nil p2) (setq p2 p1))
   (setq csc_factor (getreal "\nFator de Escala: "))
   (command  "._copy" ss1 "" "_non" p1 "_non" p1  "._move" "_p" "" "_non" p1 "_non" p2 "._scale" ss1 "" "_non" p2 csc_factor)
  )
 )
  (princ)
)

;funo Scale - Circle
(defun c:scc ()
  (setq s1 (ssget '((0 . "CIRCLE")))
	s1_len (sslength s1)
	counter 0)
  (initget 128)
  (setq scc_txfactor (getstring "\nReferncia/<Fator de Escala>... "))
  (if (not (setq scc_factor (distof scc_txfactor)))
    (progn
      (if (= "R" (strcase scc_txfactor))
	(setq scc_refdist (getdist "\nValor de Referncia...: ")
	      scc_newdist (getdist "\nNovo Valor...: ")
	      scc_factor (/ scc_refdist scc_newdist))
	(progn
	  (alert "Valor invlido!\nEntre com o fator de escala,\nou digite 'R' para referncia!")
	  (exit)
	)
      )
     )
  )
  (repeat s1_len
    (setq e1 (ssname s1 counter)
	  e1_list (entget e1)
	  e1_radius (cdr (assoc 40 e1_list))
	  e1_list (entmod (subst (cons 40 (* e1_radius scc_factor)) (assoc 40 e1_list) e1_list)))
    (setq counter (1+ counter))
  )
  (princ)
)


;Funcao MOS - Multiple offset
(defun c:mos ()
  (setq ent1 (entsel))
  (setq pt1 (getpoint "\nPonto para offset..."))
  (command "._offset" "_t" ent1 pt1 "")
  (while (/= nil (setq pt1 (getpoint "\nPonto para offset...")))
    (command "._offset" "_t" ent1 pt1 "")

  )
  (princ)
)


;*************************************************************************************************
;*************************************************************************************************
;Polylines
;*************************************************************************************************
;*************************************************************************************************


;linemend
(defun c:linemend ( / ent1 ent2 params1 1pt10 2pt10 old10
                      1pt11 2pt11 old11 dist1 dist2 dist3 dist4
                      distlist maxdist pt1 pt10 pt11)
   (setq ent1 (entsel "\nSelecione a 1a linha:  "))
   (setvar "cmdecho" 0)
   (setvar "orthomode" 1)
   (while ent1
      (setq ent1 (car ent1))
      (redraw ent1 3)
      (setq ent2 (car (entsel "\nSelecione a 2a linha...  ")))
      (while (null ent2)
         (prompt "\nNo entity found.")
         (setq ent1 (car (entsel "\nSelect second line:  ")))
         (if (/= (cdr (assoc 0 (entget ent2))) "LINE")
            (progn
               (prompt "\nLines Only!")
               (setq ent2 nil)
            )                                         ;progn
         )                                            ;if
      )                                               ;while

      (redraw ent2 3)
      (setq params1 (entget ent1)
            1pt10 (cdr (setq old10 (assoc 10 params1)))
            1pt11 (cdr (setq old11 (assoc 11 params1)))
            params2 (entget ent2)
            2pt10 (cdr (assoc 10 params2))
            2pt11 (cdr (assoc 11 params2))
            dist1 (distance 1pt10 2pt10)
            dist2 (distance 1pt10 2pt11)
            dist3 (distance 1pt11 2pt10)
            dist4 (distance 1pt11 2pt11)
            distlist
              (list
                 (list dist1 1pt10 2pt10)
                 (list dist2 1pt10 2pt11)
                 (list dist3 1pt11 2pt10)
                 (list dist4 1pt11 2pt11)
              );list
            maxdist (max dist1 dist2 dist3 dist4)
            pts (cdr (assoc maxdist distlist))
            pt10 (car pts)
            pt11 (cadr pts)
            params1 (subst (cons 10 pt10) old10 params1)
            params1 (subst (cons 11 pt11) old11 params1)
      )                                               ;setq calculations

      (entdel ent2)
      (entmod params1)
      (entupd ent1)
      (setq ent1 (entsel "\nSelect first line:  "))
   )                                                  ;while loop

   (setvar "cmdecho" 1)
   (gc)
  (princ)
)                                                     ;defun



;altera espassura de vrias polilinhas
(defun c:plwid ()
  (command "._undo" "_m")
  (setq new_width (getreal "\nEspessura desejada..."))
  (setq s1 (ssget))
  (setq s1_n (sslength s1)
	counter 0
  )
  (repeat s1_n
    (setq ent (ssname s1 counter))
    (setq ent1 (entget ent))
    (if (or (= "LWPOLYLINE" (cdr (assoc 0 ent1)))
	    (= "POLYLINE" (cdr (assoc 0 ent1)))
	)
      (command "._pedit" ent "_w" new_width "")
    )
    (setq counter (1+ counter))
  )
  (princ)
)

;spline para pline via divide (quantidade)
(defun c:spldv ()
  (guarda '("blipmode" "cecolor" "celtype" "clayer" "cmdecho" "orthomode"
      "textstyle" "luprec" "osmode" ))
  (setq osmo (getvar "osmode"))
  (setvar "osmode" 0)
  (setq cmdecho (getvar "CMDECHO"))
  (setvar "CMDECHO" 0)
  (setq blipmode (getvar "BLIPMODE"))
  (setvar "BLIPMODE" 0)
  (setq snapmode (getvar "SNAPMODE"))
  (setvar "SNAPMODE" 0)
  (setq entpick (entsel "\nSelecione uma SPLINE/3DPOLY: "))
  (setq ent (entget (car entpick)))
  (setq sp (cdr (assoc 10 ent)))
  (setq ep (cdr (assoc 10 (reverse ent))))
  (setq spl_dist (getint "\nEntre com o numero de segmentos: "))
  (setq clayer (getvar "CLAYER"))
  (cn7_verylayer "spl" 214)  

  (princ "\nAgora, espere...")
  (command "_.divide" entpick spl_dist)
  (setq	aws (ssget "X"
		   (list (cons 0 "POINT") (cons 8 "SPL"))
	    )
  )
  (princ ".")
  (setq number (sslength aws))
  (setq count 0)
  (command "_.pline" ep)
  (princ ".")
  (while (< count number)    (setq entnam (ssname aws count))
    (setq ent (entget entnam))
    (setq ins (cdr (assoc 10 ent)))
    (command ins)
    (princ ".")
    (setq count (1+ count))
  )
  (command sp "")
  (command "_.erase" aws entpick "")
  (devolve)
  (princ)
)


;spline para pline via measure (tamanho)
(defun c:splmu ()
  (guarda '("blipmode" "cecolor" "celtype" "clayer" "cmdecho" "orthomode"
      "textstyle" "luprec" "osmode" ))
  (setq osmo (getvar "osmode"))
  (setvar "osmode" 0)
  (setq cmdecho (getvar "CMDECHO"))
  (setvar "CMDECHO" 0)
  (setq blipmode (getvar "BLIPMODE"))
  (setvar "BLIPMODE" 0)
  (setq snapmode (getvar "SNAPMODE"))
  (setvar "SNAPMODE" 0)
  (setq entpick (entsel "\nSelecione uma SPLINE/3DPOLY: "))
  (setq ent (entget (car entpick)))
  (setq sp (cdr (assoc 10 ent)))
  (setq ep (cdr (assoc 10 (reverse ent))))
  (setq spl_dist (getreal "\nEntre com o tamanho dos segmentos: "))
  (setq clayer (getvar "CLAYER"))
  (cn7_verylayer "spl" 214)
  (princ "\nAgora, espere...")
  (command "_.measure" entpick spl_dist)
  (setq	aws (ssget "X"
		   (list (cons 0 "POINT") (cons 8 "SPL"))
	    )
  )  (princ ".")
  (setq number (sslength aws))
  (setq count 0)
  (command "_.pline" ep)
  (princ ".")
  (while (< count number)
    (setq entnam (ssname aws count))
    (setq ent (entget entnam))
    (setq ins (cdr (assoc 10 ent)))
    (command ins)
    (princ ".")
    (setq count (1+ count))  )
  (command sp "")
  (command "_.erase" aws entpick "")
  (devolve)
  (princ "\nFeito!")
  (princ)
)



;****************************************************************
;         Interpolacao Topografica
;         Para Confeccao de Curvas de Niveis
;****************************************************************

;***********  Interpolao de 1 curva no meio de outras duas   *************
(defun c:it1 ()
  (setq osmo (getvar "osmode"))
  (setq laymo (getvar "clayer"))
  (cn7_verylayer "Top_curvas_interpoladas" 1)
  (setvar "osmode" 512)
  (setq pt1 (getpoint "\nPrimeiro ponto..."))
  (setq next_pto (getpoint pt1 "\nPrximo ponto..."))
  (setq pt0 (polar pt1 (angle pt1 next_pto) (/ (distance pt1 next_pto) 2)))
  (command "._pline" "_non" pt0)
  (keep)
  (decide)
  (princ)
)


(defun keep ()
	(while (/= nil (setq pt1 (getpoint "\nPrimeiro ponto...")))
	    (progn
  		(setq next_pto (getpoint pt1 "\nPrximo ponto..."))
  		(setq pt0 (polar pt1 (angle pt1 next_pto) (/ (distance pt1 next_pto) 2)))
	  	(command "_non" pt0)
	    );progn
	);while
  	(decide)
);defun


(defun decide ()
  (initget 1 "Undo eXit Continue")
  (setq resp (getkword "\neXit/Undo/Continue <X/U/C>: "))
  (if (= "EXIT" (strcase resp))
    (progn
      (command "")
      (setvar "osmode" osmo)
      (setvar "clayer" laymo)
      (command "._cmdecho" 0)
      (devolve)
      (exit)
      )
    (progn
      (if (= "UNDO" (strcase resp))
	(progn
	  (command "_u")
	  (keep)
	  )
	(keep)
      )
    )
  )
)


;***********  Interpolao de n curvas no meio de outras duas   *************
(defun c:curvas () (setq n_curv (1+ (getint "\nNmero de curvas a interpolar..."))) (princ))

(defun c:itn ()
  	(if (= nil n_curv) (setq n_curv (1+ (getint "\nNmero de curvas a interpolar..."))))
  	(setq osmo (getvar "osmode"))
  	(setq laymo (getvar "osmode"))
  	(cn7_verylayer "Top_curvas_interpoladas" 1)
  	(setvar "osmode" 512)
  	(while (/= nil (setq pt1 (getpoint "\nPrimeiro ponto...<ENTER> para terminar...")))
	  (setq next_pto (getpoint pt1 "\nPrximo ponto..."))
	  (command "._line" pt1 next_pto "")
	  (setq e1 (entlast))
	  (command "._divide" (entlast) n_curv "._erase" e1 "")
	)
  	(setvar "osmode" osmo)
  	(setvar "clayer" laymo)
  (princ)
)

